"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _stringify = _interopRequireDefault(require("@babel/runtime/core-js/json/stringify"));

var _keys = _interopRequireDefault(require("@babel/runtime/core-js/object/keys"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _assert = _interopRequireDefault(require("assert"));

var _path = require("path");

var _mkdirp = require("mkdirp");

var _fs = require("fs");

var _chokidar = _interopRequireDefault(require("chokidar"));

var _chalk = _interopRequireDefault(require("chalk"));

var _lodash = _interopRequireDefault(require("lodash.debounce"));

var _getRouteConfig = _interopRequireDefault(require("./getRouteConfig"));

var _requestCache = require("./requestCache");

var _winPath = _interopRequireDefault(require("./winPath"));

var _normalizeEntry = _interopRequireDefault(require("./normalizeEntry"));

var _constants = require("./constants");

var debug = require('debug')('umi:FilesGenerator');

var FilesGenerator =
/*#__PURE__*/
function () {
  function FilesGenerator(service) {
    (0, _classCallCheck2.default)(this, FilesGenerator);
    this.service = service;
    this.routesContent = null;
    this.hasRebuildError = false;
  }

  (0, _createClass2.default)(FilesGenerator, [{
    key: "generate",
    value: function generate() {
      var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var paths = this.service.paths;
      var absTmpDirPath = paths.absTmpDirPath,
          tmpDirPath = paths.tmpDirPath;
      debug("Mkdir tmp dir: ".concat(tmpDirPath));
      (0, _mkdirp.sync)(absTmpDirPath);
      this.generateFiles();
      if (opts.onChange) opts.onChange();
    }
  }, {
    key: "createWatcher",
    value: function createWatcher(path) {
      var _this = this;

      var watcher = _chokidar.default.watch(path, {
        ignored: /(^|[\/\\])\../,
        // ignore .dotfiles
        ignoreInitial: true
      });

      watcher.on('all', (0, _lodash.default)(function (event, path) {
        debug("".concat(event, " ").concat(path));

        _this.rebuild();
      }, 100));
      return watcher;
    }
  }, {
    key: "watch",
    value: function watch() {
      var _this2 = this;

      var paths = this.service.paths;
      var watcherPaths = this.service.applyPlugins('modifyPageWatchers', {
        initialValue: [paths.absPagesPath]
      });
      this.watchers = watcherPaths.map(function (p) {
        return _this2.createWatcher(p);
      });
      process.on('SIGINT', function () {
        _this2.unwatch();
      });
    }
  }, {
    key: "unwatch",
    value: function unwatch() {
      if (this.watchers) {
        this.watchers.forEach(function (watcher) {
          watcher.close();
        });
      }
    }
  }, {
    key: "rebuild",
    value: function rebuild() {
      var devServer = this.service.devServer;

      try {
        // rebuild 时只生成 router.js
        this.generateRouterJS();
        if (this.onChange) this.onChange();

        if (this.hasRebuildError) {
          // 从出错中恢复时，刷新浏览器
          devServer.sockWrite(devServer.sockets, 'content-changed');
          this.hasRebuildError = false;
        }
      } catch (e) {
        // 向浏览器发送出错信息
        devServer.sockWrite(devServer.sockets, 'errors', [e.message]);
        this.hasRebuildError = true;
        this.routesContent = null; // why?

        debug("Generate failed: ".concat(e.message));
        debug(e);
        console.error(_chalk.default.red(e.message));
      }
    }
  }, {
    key: "generateFiles",
    value: function generateFiles() {
      var _service = this.service,
          paths = _service.paths,
          entryJSTpl = _service.entryJSTpl,
          config = _service.config;
      this.service.applyPlugins('generateFiles');
      this.generateRouterJS(); // Generate umi.js

      var entryContent = (0, _fs.readFileSync)(entryJSTpl || paths.defaultEntryTplPath, 'utf-8');
      entryContent = this.service.applyPlugins('modifyEntryFile', {
        initialValue: entryContent
      });
      entryContent = entryContent.replace(_constants.PLACEHOLDER_IMPORT, '');
      entryContent = entryContent.replace(_constants.PLACEHOLDER_RENDER, "ReactDOM.render(React.createElement(require('./router').default), document.getElementById('root'));");

      if (!config.disableServiceWorker) {
        entryContent = "".concat(entryContent, "\n// Enable service worker\nif (process.env.NODE_ENV === 'production') {\n  require('./registerServiceWorker');\n}\n      ");
      }

      (0, _fs.writeFileSync)(paths.absLibraryJSPath, entryContent, 'utf-8'); // Generate registerServiceWorker.js

      (0, _fs.writeFileSync)(paths.absRegisterSWJSPath, (0, _fs.readFileSync)(paths.defaultRegisterSWTplPath), 'utf-8');
    }
  }, {
    key: "generateRouterJS",
    value: function generateRouterJS() {
      var _service2 = this.service,
          paths = _service2.paths,
          config = _service2.config;
      var absRouterJSPath = paths.absRouterJSPath;
      var routes = (0, _getRouteConfig.default)(paths, config);
      this.service.setRoutes(routes);
      var routesContent = this.getRouterJSContent(); // 避免文件写入导致不必要的 webpack 编译

      if (this.routesContent !== routesContent) {
        (0, _fs.writeFileSync)(absRouterJSPath, routesContent, 'utf-8');
        this.routesContent = routesContent;
      }
    }
  }, {
    key: "getRouterJSContent",
    value: function getRouterJSContent() {
      var _service3 = this.service,
          routerTpl = _service3.routerTpl,
          paths = _service3.paths,
          libraryName = _service3.libraryName;
      var routerTplPath = routerTpl || paths.defaultRouterTplPath;
      (0, _assert.default)((0, _fs.existsSync)(routerTplPath), "routerTpl don't exists: ".concat(routerTplPath));
      var tplContent = (0, _fs.readFileSync)(routerTplPath, 'utf-8');
      tplContent = this.service.applyPlugins('modifyRouterFile', {
        initialValue: tplContent
      });
      var routerContent = this.service.applyPlugins('modifyRouterContent', {
        initialValue: this.getRouterContent()
      });
      return tplContent.replace('<%= IMPORT %>', '').replace('<%= ROUTER %>', routerContent).replace(/<%= libraryName %>/g, libraryName);
    }
  }, {
    key: "getRouterContent",
    value: function getRouterContent() {
      var _this3 = this;

      var _service4 = this.service,
          routes = _service4.routes,
          config = _service4.config,
          paths = _service4.paths;
      var routesByPath = routes.reduce(function (memo, _ref) {
        var path = _ref.path,
            component = _ref.component;
        memo[path] = component;
        return memo;
      }, {});
      var loading = config.loading;
      var loadingOpts = '';

      if (loading) {
        loadingOpts = "loading: require('".concat((0, _winPath.default)((0, _path.join)(paths.cwd, loading)), "').default,");
      }

      var routesContent = (0, _keys.default)(routesByPath).map(function (key) {
        var pageJSFile = (0, _winPath.default)((0, _path.relative)(paths.tmpDirPath, routesByPath[key]));
        debug("requested: ".concat((0, _stringify.default)((0, _requestCache.getRequest)())));
        var isDev = process.env.NODE_ENV === 'development';
        var component;
        var isCompiling = false;

        if (isDev && process.env.COMPILE_ON_DEMAND !== 'none') {
          if ((0, _requestCache.getRequest)()[key]) {
            component = "require('".concat(pageJSFile, "').default");
          } else {
            component = '() => <div>Compiling...</div>';
            isCompiling = true;
          }
        } else {
          component = "dynamic(() => import(/* webpackChunkName: '".concat((0, _normalizeEntry.default)(routesByPath[key]), "' */'").concat(pageJSFile, "'), { ").concat(loadingOpts, " })");
        }

        component = _this3.service.applyPlugins('modifyRouteComponent', {
          initialValue: component,
          args: {
            isCompiling: isCompiling
          }
        });
        return "    <Route exact path=\"".concat(key, "\" component={").concat(component, "}></Route>");
      });
      return "\n<Router history={window.g_history}>\n  <Switch>\n".concat(routesContent.join('\n'), "\n  </Switch>\n</Router>\n    ").trim();
    }
  }]);
  return FilesGenerator;
}();

exports.default = FilesGenerator;