"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _getIterator2 = _interopRequireDefault(require("@babel/runtime/core-js/get-iterator"));

var _stringify = _interopRequireDefault(require("@babel/runtime/core-js/json/stringify"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _path = require("path");

var _ejs = _interopRequireDefault(require("ejs"));

var _mkdirp = require("mkdirp");

var _assert = _interopRequireDefault(require("assert"));

var _fs = require("fs");

var _normalizeEntry = _interopRequireDefault(require("./normalizeEntry"));

var debug = require('debug')('umi:HtmlGenerator');

var HtmlGenerator =
/*#__PURE__*/
function () {
  function HtmlGenerator(service) {
    var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    (0, _classCallCheck2.default)(this, HtmlGenerator);
    this.service = service;
    this.chunksMap = opts.chunksMap;
  }
  /*
  // e.g.
  //
  // path  no htmlSuffix     with htmlSuffix
  // ---
  // /     /index.html       /index.html
  // /a    /a/index.html     /a.html
  // /a/   /a/index.html     /a.html
  // /a/b  /a/b/index.html   /a/b.html
  */


  (0, _createClass2.default)(HtmlGenerator, [{
    key: "getHtmlPath",
    value: function getHtmlPath(path) {
      var config = this.service.config;
      var htmlSuffix = config.exportStatic && typeof config.exportStatic === 'object' && config.exportStatic.htmlSuffix;
      path = path.slice(1);

      if (path === '') {
        return 'index.html';
      } // remove last slash


      path = path.replace(/\/$/, '');

      if (htmlSuffix) {
        return path;
      } else {
        return "".concat(path, "/index.html");
      }
    } // 仅在 build 时调用

  }, {
    key: "generate",
    value: function generate() {
      var _this = this;

      var _service = this.service,
          config = _service.config,
          routes = _service.routes,
          paths = _service.paths;

      if (config.exportStatic) {
        var pagesConfig = config.pages || {};
        routes.forEach(function (route) {
          var path = route.path;

          var content = _this.getContent({
            route: route,
            pageConfig: pagesConfig[path]
          });

          var outputPath = (0, _path.join)(paths.absOutputPath, _this.getHtmlPath(path));
          (0, _mkdirp.sync)((0, _path.dirname)(outputPath));
          (0, _fs.writeFileSync)(outputPath, content, 'utf-8');
        });
      } else {
        var content = this.getContent();
        var outputPath = (0, _path.join)(paths.absOutputPath, 'index.html');
        (0, _fs.writeFileSync)(outputPath, content, 'utf-8');
      }
    }
  }, {
    key: "getContent",
    value: function getContent() {
      var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var _opts$pageConfig = opts.pageConfig,
          pageConfig = _opts$pageConfig === void 0 ? {} : _opts$pageConfig,
          _opts$route = opts.route,
          route = _opts$route === void 0 ? {} : _opts$route;
      var _service2 = this.service,
          paths = _service2.paths,
          webpackConfig = _service2.webpackConfig;
      var document = pageConfig.document,
          context = pageConfig.context; // e.g.
      // path: /user.html
      // component: ./user/page.js
      // entry: ./user

      var path = route.path,
          component = route.component;
      var customDocPath = document ? (0, _path.join)(paths.cwd, document) : paths.absPageDocumentPath;
      var docPath = (0, _fs.existsSync)(customDocPath) ? customDocPath : paths.defaultDocumentPath;
      var tpl = (0, _fs.readFileSync)(docPath, 'utf-8');

      var html = _ejs.default.render(tpl, context, {
        _with: false,
        localsName: 'context'
      });

      var relPath = path ? new Array(path.slice(1).split(_path.sep).length).join('../') : '';
      relPath = relPath === '' ? './' : relPath; // set publicPath

      var publicPath = webpackConfig.output.publicPath;
      publicPath = makeSureHaveLastSlash(publicPath);
      var resourceBaseUrl = "'".concat(publicPath, "'");
      var pathToScript = publicPath;
      debug("publicPath: ".concat(publicPath));

      if (!(publicPath.charAt(0) === '/' || publicPath.indexOf('http://') === 0 || publicPath.indexOf('https://') === 0 ||
      /* 变量 */
      publicPath === '{{ publicPath }}')) {
        // 相对路径时需和 routerBase 匹配使用，否则子文件夹路由会出错
        resourceBaseUrl = "location.origin + window.routerBase + '".concat(stripFirstSlash(publicPath), "'");
        pathToScript = "".concat(relPath).concat(publicPath);
      }

      function getAssetsPath(file) {
        return "".concat(pathToScript).concat(stripFirstSlash(file)).replace(/^\.\/\.\//, './');
      }

      var routerBase;

      if (process.env.BASE_URL) {
        routerBase = (0, _stringify.default)(process.env.BASE_URL);
      } else {
        routerBase = path ? "location.pathname.split('/').slice(0, -".concat(path.split('/').length - 1, ").concat('').join('/')") : "'/'";
      }

      var inlineScriptContent = "\n<script>\n  window.routerBase = ".concat(routerBase, ";\n  window.resourceBaseUrl = ").concat(resourceBaseUrl, ";\n</script>\n    ").trim();
      inlineScriptContent = this.service.applyPlugins('modifyHTMLScript', {
        initialValue: inlineScriptContent
      });
      var isDev = process.env.NODE_ENV === 'development';
      var cssFiles = isDev ? [] : this.getCSSFiles(component);
      var cssContent = cssFiles.map(function (file) {
        return "<link rel=\"stylesheet\" href=\"".concat(getAssetsPath(file), "\" />");
      }).join('\r\n');
      var jsFiles = this.getJSFiles(component);
      var jsContent = jsFiles.map(function (file) {
        return "<script src=\"".concat(getAssetsPath(file), "\"></script>");
      }).join('\r\n');
      var injectContent = "\n".concat(cssContent, "\n").concat(inlineScriptContent, "\n").concat(jsContent, "\n    ").trim();
      html = html.replace('</body>', "".concat(injectContent, "\r\n</body>")); // 插件最后处理一遍 HTML

      html = this.service.applyPlugins('modifyHTML', {
        initialValue: html,
        args: {
          route: route
        }
      });
      return "".concat(html, "\r\n");
    }
  }, {
    key: "getJSFiles",
    value: function getJSFiles(component) {
      var _service3 = this.service,
          libraryName = _service3.libraryName,
          config = _service3.config;
      var files = [];

      try {
        files.push(this.getFile(libraryName, '.js'));
      } catch (e) {}

      var isDev = process.env.NODE_ENV === 'development';

      if (!isDev && config.exportStatic) {
        try {
          files.push(this.getFile("__common-".concat(libraryName), '.js'));
        } catch (e) {}

        try {
          if (component) {
            files.push(this.getFile((0, _normalizeEntry.default)(component), '.js'));
          }
        } catch (e) {}
      }

      return files;
    }
  }, {
    key: "getCSSFiles",
    value: function getCSSFiles(component) {
      var libraryName = this.service.libraryName;
      var files = [];

      try {
        files.push(this.getFile(libraryName, '.css'));
      } catch (e) {}

      try {
        if (component) {
          files.push(this.getFile((0, _normalizeEntry.default)(component), '.css'));
        }
      } catch (e) {}

      return files;
    }
  }, {
    key: "getFile",
    value: function getFile(name, type) {
      if (!this.chunksMap) {
        return [name, type].join('');
      }

      var files = this.chunksMap[name];
      (0, _assert.default)(files, "name ".concat(name, " don't exists in chunksMap ").concat((0, _stringify.default)(this.chunksMap)));
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator2.default)(files), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var _file = _step.value;

          if ((0, _path.extname)(_file) === type) {
            return _file;
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      throw new Error("getFile failed:\nmap: ".concat((0, _stringify.default)(this.chunksMap), "\nname: ").concat(name, "\ntype: ").concat(type));
    }
  }]);
  return HtmlGenerator;
}();

exports.default = HtmlGenerator;

function stripFirstSlash(str) {
  return str.replace(/^\//, '');
}

function makeSureHaveLastSlash(str) {
  if (str === '{{ publicPath }}' || str.slice(-1) === '/') {
    return str;
  } else {
    return "".concat(str, "/");
  }
}