"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _promise = _interopRequireDefault(require("@babel/runtime/core-js/promise"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _rimraf = require("rimraf");

var _fs = require("fs");

var _path = require("path");

var _getUserConfig = _interopRequireWildcard(require("af-webpack/getUserConfig"));

var _reactDevUtils = require("af-webpack/react-dev-utils");

var _chalk = _interopRequireDefault(require("chalk"));

var _getPaths = _interopRequireDefault(require("./getPaths"));

var _getRouteConfig = _interopRequireDefault(require("./getRouteConfig"));

var _registerBabel = require("./registerBabel");

var _watch = require("./getConfig/watch");

var _UserConfig = _interopRequireDefault(require("./UserConfig"));

var _getPlugins = _interopRequireDefault(require("./getPlugins"));

var _getWebpackConfig = _interopRequireDefault(require("./getWebpackConfig"));

var _chunksToMap = _interopRequireDefault(require("./utils/chunksToMap"));

var _send = _interopRequireWildcard(require("./send"));

var _FilesGenerator = _interopRequireDefault(require("./FilesGenerator"));

var _HtmlGenerator = _interopRequireDefault(require("./HtmlGenerator"));

var _createRouteMiddleware = _interopRequireDefault(require("./createRouteMiddleware"));

var _PluginAPI = _interopRequireDefault(require("./PluginAPI"));

var debug = require('debug')('umi-build-dev:Service');

var Service =
/*#__PURE__*/
function () {
  function Service(cwd, _ref) {
    var _this = this;

    var pluginFiles = _ref.plugins,
        babel = _ref.babel,
        entryJSTpl = _ref.entryJSTpl,
        routerTpl = _ref.routerTpl,
        hash = _ref.hash,
        preact = _ref.preact,
        extraResolveModules = _ref.extraResolveModules,
        _ref$libraryName = _ref.libraryName,
        libraryName = _ref$libraryName === void 0 ? 'umi' : _ref$libraryName,
        _ref$staticDirectory = _ref.staticDirectory,
        staticDirectory = _ref$staticDirectory === void 0 ? 'static' : _ref$staticDirectory,
        _ref$tmpDirectory = _ref.tmpDirectory,
        tmpDirectory = _ref$tmpDirectory === void 0 ? '.umi' : _ref$tmpDirectory,
        _ref$outputPath = _ref.outputPath,
        outputPath = _ref$outputPath === void 0 ? './dist' : _ref$outputPath;
    (0, _classCallCheck2.default)(this, Service);

    this.reload = function () {
      if (!_this.devServer) return;

      _this.devServer.sockWrite(_this.devServer.sockets, 'content-changed');
    };

    this.printWarn = function (messages) {
      if (!_this.devServer) return;
      messages = typeof messages === 'string' ? [messages] : messages;

      _this.devServer.sockWrite(_this.devServer.sockets, 'warns', messages);
    };

    this.printError = function (messages) {
      if (!_this.devServer) return;
      messages = typeof messages === 'string' ? [messages] : messages;

      _this.devServer.sockWrite(_this.devServer.sockets, 'errors', messages);
    };

    this.restart = function (why) {
      if (!_this.devServer) return;
      (0, _reactDevUtils.clearConsole)();
      console.log(_chalk.default.green("Since ".concat(why, ", try to restart server")));
      (0, _watch.unwatch)();

      _this.devServer.close();

      process.send({
        type: 'RESTART'
      });
    };

    this.cwd = cwd || process.cwd();
    this.pluginFiles = pluginFiles;
    this.babel = babel;
    this.entryJSTpl = entryJSTpl;
    this.routerTpl = routerTpl;
    this.hash = hash;
    this.preact = preact;
    this.extraResolveModules = extraResolveModules;
    this.libraryName = libraryName;
    this.staticDirectory = staticDirectory;
    this.tmpDirectory = tmpDirectory;
    this.outputPath = outputPath;
    this.paths = (0, _getPaths.default)(this);
    this.pluginMethods = {};
    this.registerBabel();
  }

  (0, _createClass2.default)(Service, [{
    key: "registerBabel",
    value: function registerBabel() {
      (0, _registerBabel.registerBabelForConfig)(this.babel, {
        paths: this.paths
      });
    }
  }, {
    key: "setRoutes",
    value: function setRoutes(routes) {
      this.routes = routes;
    }
  }, {
    key: "getWebpackRCConfig",
    value: function getWebpackRCConfig() {
      return (0, _getUserConfig.default)({
        cwd: this.cwd,
        disabledConfigs: ['entry', 'outputPath', 'hash']
      });
    }
  }, {
    key: "dev",
    value: function dev() {
      var _this2 = this;

      // 获取 .webpackrc 配置
      var returnedWatchWebpackRCConfig = null;

      try {
        var configObj = this.getWebpackRCConfig();
        this.webpackRCConfig = configObj.config;
        returnedWatchWebpackRCConfig = configObj.watch;
      } catch (e) {
        console.error(_chalk.default.red(e.message));
        debug('Get .webpackrc config failed, watch config and reload'); // 监听配置项变更，然后重新执行 dev 逻辑

        (0, _getUserConfig.watchConfigs)().on('all', function (event, path) {
          debug("[".concat(event, "] ").concat(path, ", unwatch and reload")); // 重新执行 dev 逻辑

          (0, _getUserConfig.unwatchConfigs)();

          _this2.dev();
        });
        return;
      } // 获取用户 config.js 配置


      var userConfig = new _UserConfig.default(this);

      try {
        this.config = userConfig.getConfig({
          force: true
        });
      } catch (e) {
        console.error(_chalk.default.red(e.message));
        debug('Get config failed, watch config and reload'); // 监听配置项变更，然后重新执行 dev 逻辑

        userConfig.watchConfigs(function (event, path) {
          debug("[".concat(event, "] ").concat(path, ", unwatch and reload")); // 重新执行 dev 逻辑

          userConfig.unwatch();

          _this2.dev();
        });
        return;
      }

      this.initPlugins();
      this.initRoutes(); // 生成入口文件

      var filesGenerator = new _FilesGenerator.default(this);

      try {
        filesGenerator.generate({
          onChange: function onChange() {
            _this2.sendPageList();
          }
        });
      } catch (e) {
        console.error(_chalk.default.red(e.message));
        console.error(_chalk.default.red(e.stack));
        debug('Generate entry failed, watch pages and reload');
        filesGenerator.watch({
          onChange: function onChange() {
            filesGenerator.unwatch();

            _this2.dev();
          }
        });
        return;
      }

      var webpackConfig = (0, _getWebpackConfig.default)(this);
      this.webpackConfig = webpackConfig;
      var isCompileDone = false;

      var _onCompileDone = function onCompileDone() {
        isCompileDone = true;
      };

      var extraMiddlewares = this.applyPlugins('modifyMiddlewares', {
        initialValue: [(0, _createRouteMiddleware.default)(this, {
          rebuildEntry: function rebuildEntry() {
            if (!isCompileDone) {
              // 改写
              var defaultOnCompileDone = _onCompileDone;

              _onCompileDone = function onCompileDone() {
                debug('new compile done');
                filesGenerator.rebuild();
                defaultOnCompileDone(); // 再次改写

                _onCompileDone = function onCompileDone() {};
              };
            } else {
              filesGenerator.rebuild();
            }
          }
        })]
      });

      require('af-webpack/dev').default({
        webpackConfig: webpackConfig,
        extraMiddlewares: extraMiddlewares,
        beforeServer: function beforeServer(devServer) {
          _this2.applyPlugins('beforeServer', {
            args: {
              devServer: devServer
            }
          });
        },
        afterServer: function afterServer(devServer) {
          _this2.devServer = devServer;

          _this2.applyPlugins('afterServer', {
            args: {
              devServer: devServer
            }
          });

          returnedWatchWebpackRCConfig(devServer, {
            beforeChange: function beforeChange() {
              _this2.registerBabel();
            }
          });
          userConfig.setConfig(_this2.config);
          userConfig.watchWithDevServer();
          filesGenerator.watch();
        },
        onCompileDone: function onCompileDone(stats) {
          _onCompileDone();

          _this2.applyPlugins('onCompileDone', {
            args: {
              stats: stats
            }
          });
        },
        proxy: this.webpackRCConfig.proxy || {},
        // 支付宝 IDE 里不自动打开浏览器
        openBrowser: !process.env.ALIPAY_EDITOR
      });
    }
  }, {
    key: "initRoutes",
    value: function initRoutes() {
      this.routes = this.applyPlugins('modifyRoutes', {
        initialValue: (0, _getRouteConfig.default)(this.paths, this.config)
      });
    }
  }, {
    key: "initPlugins",
    value: function initPlugins() {
      var _this3 = this;

      // TODO: getPlugins 的出错处理
      this.plugins = (0, _getPlugins.default)({
        configPlugins: this.config.plugins,
        pluginsFromOpts: this.pluginFiles,
        cwd: this.cwd,
        babel: this.babel
      });
      this.plugins.forEach(function (_ref2) {
        var id = _ref2.id,
            apply = _ref2.apply;

        try {
          apply(new _PluginAPI.default(id, _this3));
        } catch (e) {
          console.error(_chalk.default.red("Plugin ".concat(id, " initialize failed, ").concat(e.message)));
          console.error(e);
          process.exit(1);
        }
      });
      this.applyPlugins('onStart');
    }
  }, {
    key: "applyPlugins",
    value: function applyPlugins(key) {
      var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      return (this.pluginMethods[key] || []).reduce(function (memo, _ref3) {
        var fn = _ref3.fn;
        return fn({
          memo: memo,
          args: opts.args
        });
      }, opts.initialValue);
    }
  }, {
    key: "sendPageList",
    value: function sendPageList() {
      var pageList = this.routes.map(function (route) {
        return {
          path: route.path
        };
      });
      (0, _send.default)({
        type: _send.PAGE_LIST,
        payload: pageList
      });
    }
  }, {
    key: "build",
    value: function build() {
      var _this4 = this;

      this.webpackRCConfig = this.getWebpackRCConfig().config;
      var userConfig = new _UserConfig.default(this);
      this.config = userConfig.getConfig();
      this.initPlugins();
      this.initRoutes();
      debug("Clean tmp dir ".concat(this.paths.tmpDirPath));
      (0, _rimraf.sync)(this.paths.absTmpDirPath);
      debug("Clean output path ".concat(this.paths.outputPath));
      (0, _rimraf.sync)(this.paths.absOutputPath);
      debug('Generate entry');
      var filesGenerator = new _FilesGenerator.default(this);
      filesGenerator.generate();
      var webpackConfig = (0, _getWebpackConfig.default)(this);
      this.webpackConfig = webpackConfig;
      return new _promise.default(function (resolve) {
        require('af-webpack/build').default({
          webpackConfig: webpackConfig,
          success: function success(_ref4) {
            var stats = _ref4.stats;

            if (process.env.RM_TMPDIR === 'none') {
              debug("Clean tmp dir ".concat(_this4.paths.tmpDirPath));
              (0, _rimraf.sync)(_this4.paths.absTmpDirPath);
            }

            debug("Bundle html files");
            var chunksMap = (0, _chunksToMap.default)(stats.compilation.chunks);

            try {
              var hg = new _HtmlGenerator.default(_this4, {
                chunksMap: chunksMap
              });
              hg.generate();
            } catch (e) {
              console.log(e);
            }

            debug('Move service-worker.js');
            var sourceSW = (0, _path.join)(_this4.paths.absOutputPath, _this4.staticDirectory, 'service-worker.js');
            var targetSW = (0, _path.join)(_this4.paths.absOutputPath, 'service-worker.js');

            if ((0, _fs.existsSync)(sourceSW)) {
              (0, _fs.renameSync)(sourceSW, targetSW);
            }

            _this4.applyPlugins('buildSuccess');

            (0, _send.default)({
              type: _send.BUILD_DONE
            });
            resolve();
          }
        });
      });
    }
  }]);
  return Service;
}();

exports.default = Service;