"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _getIterator2 = _interopRequireDefault(require("@babel/runtime/core-js/get-iterator"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _keys = _interopRequireDefault(require("@babel/runtime/core-js/object/keys"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _path = require("path");

var _fs = require("fs");

var _requireindex = _interopRequireDefault(require("requireindex"));

var _chalk = _interopRequireDefault(require("chalk"));

var _didyoumean = _interopRequireDefault(require("didyoumean"));

var _lodash = _interopRequireDefault(require("lodash.isequal"));

var _constants = require("./constants");

var _watch = require("./getConfig/watch");

var _createRouteMiddleware = require("./createRouteMiddleware");

var UserConfig =
/*#__PURE__*/
function () {
  function UserConfig(service) {
    (0, _classCallCheck2.default)(this, UserConfig);
    this.service = service;
    this.configFailed = false;
    this.config = null;
    this.file = null;
    this.relativeFile = null;
    this.watch = _watch.watch;
    this.unwatch = _watch.unwatch;
    this.initConfigPlugins();
  }

  (0, _createClass2.default)(UserConfig, [{
    key: "initConfigPlugins",
    value: function initConfigPlugins() {
      var _this = this;

      var map = (0, _requireindex.default)((0, _path.join)(__dirname, 'getConfig/configPlugins'));
      var plugins = (0, _keys.default)(map).map(function (key) {
        return map[key].default;
      });
      plugins = this.service.applyPlugins('modifyConfigPlugins', {
        initialValue: plugins
      });
      this.plugins = plugins.map(function (p) {
        return p(_this);
      });
    }
  }, {
    key: "getConfigFile",
    value: function getConfigFile() {
      var _service = this.service,
          paths = _service.paths,
          printWarn = _service.printWarn;

      var files = _constants.CONFIG_FILES.map(function (file) {
        return (0, _path.join)(paths.cwd, file);
      }).filter(function (file) {
        return (0, _fs.existsSync)(file);
      });

      if (files.length > 1) {
        printWarn("Muitiple config files ".concat(files.join(', '), " detected, umi will use ").concat(files[0], "."));
      }

      return files[0];
    }
  }, {
    key: "normalizeConfig",
    value: function normalizeConfig(config) {
      config = config.default || config;

      if (config.context && config.pages) {
        (0, _keys.default)(config.pages).forEach(function (key) {
          var page = config.pages[key];
          page.context = (0, _extends2.default)({}, config.context, page.context);
        });
      } // pages 配置补丁
      // /index -> /index.html
      // index -> /index.html


      if (config.pages) {
        var htmlSuffix = !!(config.exportStatic && typeof config.exportStatic === 'object' && config.exportStatic.htmlSuffix);
        config.pages = (0, _keys.default)(config.pages).reduce(function (memo, key) {
          var newKey = key;

          if (htmlSuffix && newKey.slice(-5) !== '.html') {
            newKey = "".concat(newKey, ".html");
          }

          if (newKey.charAt(0) !== '/') {
            newKey = "/".concat(newKey);
          }

          memo[newKey] = config.pages[key];
          return memo;
        }, {});
      }

      return config;
    }
  }, {
    key: "getConfig",
    value: function getConfig() {
      var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var _service2 = this.service,
          paths = _service2.paths,
          printError = _service2.printError;
      var force = opts.force,
          setConfig = opts.setConfig;
      var file = this.getConfigFile();
      this.file = file;

      if (!file) {
        return {};
      } // 强制读取，不走 require 缓存


      if (force) {
        _constants.CONFIG_FILES.forEach(function (file) {
          delete require.cache[(0, _path.join)(paths.cwd, file)];
        });
      }

      this.service.registerBabel();
      var config = null;
      var relativeFile = file.replace("".concat(paths.cwd, "/"), '');
      this.relativeFile = relativeFile;

      try {
        config = require(file); // eslint-disable-line
      } catch (e) {
        var msg = "\u914D\u7F6E\u6587\u4EF6 \"".concat(relativeFile, "\" \u89E3\u6790\u51FA\u9519\uFF0C\u8BF7\u68C0\u67E5\u8BED\u6CD5\u3002\n\r\n").concat(e.toString());
        printError(msg);
        throw new Error(msg);
      }

      config = this.normalizeConfig(config); // Validate

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator2.default)(this.plugins), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var _plugin = _step.value;
          var name = _plugin.name,
              validate = _plugin.validate;

          if (config[name] && validate) {
            try {
              _plugin.validate(config[name]);
            } catch (e) {
              // 校验出错后要把值设到缓存的 config 里，确保 watch 判断时才能拿到正确的值
              if (setConfig) {
                setConfig(config);
              }

              printError(e.message);
              throw new Error("\u914D\u7F6E ".concat(name, " \u6821\u9A8C\u5931\u8D25, ").concat(e.message));
            }
          }
        } // 找下不匹配的 name

      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      var pluginNames = this.plugins.map(function (p) {
        return p.name;
      });
      (0, _keys.default)(config).forEach(function (key) {
        if (!pluginNames.includes(key)) {
          if (opts.setConfig) {
            opts.setConfig(config);
          }

          var affixmsg = "\u9009\u62E9 \"".concat(pluginNames.join(', '), "\" \u4E2D\u7684\u4E00\u9879\uFF0C\u8BE6\u89C1 https://fengdie.alipay-eco.com/doc/h5app/configuration");
          var guess = (0, _didyoumean.default)(key, pluginNames);
          var midMsg = guess ? "\u4F60\u662F\u4E0D\u662F\u60F3\u914D\u7F6E \"".concat(guess, "\" \uFF1F \u6216\u8005") : '请';

          var _msg = "\"".concat(relativeFile, "\" \u4E2D\u914D\u7F6E\u7684 \"").concat(key, "\" \u5E76\u975E\u7EA6\u5B9A\u7684\u914D\u7F6E\u9879\uFF0C").concat(midMsg).concat(affixmsg);

          printError(_msg);
          throw new Error(_msg);
        }
      });
      return config;
    }
  }, {
    key: "setConfig",
    value: function setConfig(config) {
      this.config = config;
    }
  }, {
    key: "watchWithDevServer",
    value: function watchWithDevServer() {
      var _this2 = this;

      // 配置插件的监听
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (0, _getIterator2.default)(this.plugins), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var _plugin4 = _step2.value;

          if (_plugin4.watch) {
            _plugin4.watch();
          }
        } // 配置文件的监听

      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      this.watchConfigs(function (event, path) {
        console.log("[DEBUG] [".concat(event, "] ").concat(path));

        try {
          var newConfig = _this2.getConfig({
            force: true,
            setConfig: function setConfig(newConfig) {
              console.log('set config');
              _this2.config = newConfig;
            }
          }); // 更新 middleware 的配置


          (0, _createRouteMiddleware.setConfig)(newConfig); // 从失败中恢复过来，需要 reload 一次

          if (_this2.configFailed) {
            _this2.configFailed = false;

            _this2.service.reload();
          }

          var _iteratorNormalCompletion3 = true;
          var _didIteratorError3 = false;
          var _iteratorError3 = undefined;

          try {
            for (var _iterator3 = (0, _getIterator2.default)(_this2.plugins), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
              var _plugin3 = _step3.value;
              var name = _plugin3.name;

              if (!(0, _lodash.default)(newConfig[name], _this2.config[name])) {
                if (_plugin3.onChange) {
                  _plugin3.onChange(newConfig);
                }
              }
            }
          } catch (err) {
            _didIteratorError3 = true;
            _iteratorError3 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
                _iterator3.return();
              }
            } finally {
              if (_didIteratorError3) {
                throw _iteratorError3;
              }
            }
          }

          _this2.config = newConfig;
        } catch (e) {
          _this2.configFailed = true;
          console.error(_chalk.default.red("watch handler failed, since ".concat(e.message)));
          console.error(e);
        }
      });
    }
  }, {
    key: "watchConfigs",
    value: function watchConfigs(handler) {
      return this.watch('CONFIG_FILES', _constants.CONFIG_FILES).on('all', handler);
    }
  }]);
  return UserConfig;
}();

var _default = UserConfig;
exports.default = _default;